Package: twoddpcr
Title: Classify 2-d Droplet Digital PCR (ddPCR) data and quantify the
        number of starting molecules
Version: 1.37.0
Authors@R: person("Anthony", "Chiu",
    email="anthony@achiu.me",
    role=c("aut", "cre"))
Author: Anthony Chiu [aut, cre]
Maintainer: Anthony Chiu <anthony@achiu.me>
URL: http://github.com/CRUKMI-ComputationalBiology/twoddpcr/
BugReports: http://github.com/CRUKMI-ComputationalBiology/twoddpcr/issues/
Description: The twoddpcr package takes Droplet Digital PCR (ddPCR)
    droplet amplitude data from Bio-Rad's QuantaSoft and can classify the
    droplets. A summary of the positive/negative droplet counts can be
    generated, which can then be used to estimate the number of molecules
    using the Poisson distribution. This is the first open source package
    that facilitates the automatic classification of general two channel
    ddPCR data. Previous work includes 'definetherain' (Jones et al., 2014)
    and 'ddpcRquant' (Trypsteen et al., 2015) which both handle one channel
    ddPCR experiments only. The 'ddpcr' package available on CRAN
    (Attali et al., 2016) supports automatic gating of a specific class of
    two channel ddPCR experiments only.
Depends: R (>= 3.4)
Imports: class, ggplot2, hexbin, methods, scales, shiny, stats, utils,
        RColorBrewer, S4Vectors
License: GPL-3
LazyData: true
RoxygenNote: 7.1.1
Collate: 'KRAScounts.R' 'KRASdata.R' 'global.R' 'ddpcrWell.R'
        'ddpcrPlate.R' 'clusterCentres.R' 'clusterStats.R' 'themes.R'
        'dropletPlot.R' 'exportTable.R' 'facetPlot.R' 'flatPlot.R'
        'ggplot.R' 'gridClassify.R' 'heatPlot.R' 'isTwoDimDataFrame.R'
        'kmeansClassify.R' 'knnClassify.R' 'mahRain.R' 'normalise.R'
        'numDroplets.R' 'parseCounts.R' 'plateSummary.R'
        'readCSVDataFrame.R' 'relabelClasses.R'
        'removeDropletClasses.R' 'sdRain.R' 'shinyVisApp.R'
        'shinyVisGlobal.R' 'shinyVisServer.R' 'shinyVisUI.R'
        'summaries.R' 'twoddpcr.R' 'wellNameSort.R'
Suggests: devtools, knitr, reshape2, rmarkdown, testthat, BiocStyle
VignetteBuilder: knitr
biocViews: ddPCR, Software, Classification
git_url: https://git.bioconductor.org/packages/twoddpcr
git_branch: devel
git_last_commit: 4aca120
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:43:25 UTC; biocbuild
