---
title: "twoddpcr report"
output: html_document
params:
  plateName: NA
  plate: NA
  cMethod: NA
  longMode: NA
  rainType: NA
  summaryTable: NA
---

This document is a short report generated by the [twoddpcr Shiny 
app](http://shiny.cruk.manchester.ac.uk/twoddpcr/).

**Plate name:** `r params$plateName`

**Classification method:** `r params$longMode`

**Rain type:** `r params$rainType`

## Classification plots

The combined wells in the *`r params$plateName`* plate classify as follows:

```{r warning=FALSE, message=FALSE, echo=FALSE}
dropletPlot(params$plate, cMethod=params$cMethod)
```

The individual wells can be seen as:

```{r warning=FALSE, message=FALSE, echo=FALSE}
lapply(params$plate, dropletPlot, cMethod=params$cMethod)
```

## Summary

From the above classifications, the number of positive droplets and estimated 
number of starting molecules is summarised below.

```{r echo=FALSE}
knitr::kable(params$summaryTable)
```

